# ALMA Data Reduction Script

# Calibration

thesteps = []
step_title = {0: 'Import of the ASDM',
              1: 'Fix of SYSCAL table times',
              2: 'listobs',
              3: 'A priori flagging',
              4: 'Generation and time averaging of the WVR cal table',
              5: 'Generation of the Tsys cal table',
              6: 'Generation of the antenna position cal table',
              7: 'Application of the WVR, Tsys and antpos cal tables',
              8: 'Split out science SPWs and time average',
              9: 'Listobs, and save original flags',
              10: 'Initial flagging',
              11: 'Putting a model for the flux calibrator(s)',
              12: 'Save flags before bandpass cal',
              13: 'Bandpass calibration',
              14: 'Save flags before gain cal',
              15: 'Gain calibration',
              16: 'Save flags before applycal',
              17: 'Application of the bandpass and gain cal tables',
              18: 'Split out corrected column',
              19: 'Save flags after applycal'}

if 'applyonly' not in globals(): applyonly = True
try:
  print 'List of steps to be executed ...', mysteps
  thesteps = mysteps
except:
  print 'global variable mysteps not set.'
if (thesteps==[]):
  thesteps = range(0,len(step_title))
  print 'Executing all steps: ', thesteps

# The Python variable 'mysteps' will control which steps
# are executed when you start the script using
#   execfile('scriptForCalibration.py')
# e.g. setting
#   mysteps = [2,3,4]# before starting the script will make the script execute
# only steps 2, 3, and 4
# Setting mysteps = [] will make it execute all steps.

import re

import os

if applyonly != True: es = aU.stuffForScienceDataReduction() 


if re.search('^4.7.0', casadef.casa_version) == None:
 sys.exit('ERROR: PLEASE USE THE SAME VERSION OF CASA THAT YOU USED FOR GENERATING THE SCRIPT: 4.7.0')


# CALIBRATE_AMPLI: 
# CALIBRATE_ATMOSPHERE: Arp_220,J1516+1932,J1550+0527,J1924-2914
# CALIBRATE_BANDPASS: J1924-2914
# CALIBRATE_FLUX: J1550+0527
# CALIBRATE_FOCUS: 
# CALIBRATE_PHASE: J1516+1932
# CALIBRATE_POINTING: J1550+0527,J1924-2914
# OBSERVE_CHECK: J1532+2344
# OBSERVE_TARGET: Arp_220

# Using reference antenna = DA61

# Import of the ASDM
mystep = 0
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  if os.path.exists('uid___A002_Xb56f17_X164.ms') == False:
    importasdm('uid___A002_Xb56f17_X164', asis='Antenna Station Receiver Source CalAtmosphere CalWVR CorrelatorMode SBSummary', bdfflags=True, lazy=False, process_caldevice=False)
    if not os.path.exists('uid___A002_Xb56f17_X164.ms.flagversions'):
      print 'ERROR in importasdm. Output MS is probably not useful. Will stop here.'
      thesteps = []
  if applyonly != True: es.fixForCSV2555('uid___A002_Xb56f17_X164.ms')

# Fix of SYSCAL table times
mystep = 1
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  from recipes.almahelpers import fixsyscaltimes
  fixsyscaltimes(vis = 'uid___A002_Xb56f17_X164.ms')

print "# A priori calibration"

# listobs
mystep = 2
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  os.system('rm -rf uid___A002_Xb56f17_X164.ms.listobs')
  listobs(vis = 'uid___A002_Xb56f17_X164.ms',
    listfile = 'uid___A002_Xb56f17_X164.ms.listobs')
  
  

# A priori flagging
mystep = 3
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  flagdata(vis = 'uid___A002_Xb56f17_X164.ms',
    mode = 'manual',
    spw = '5~12,17~32',
    autocorr = T,
    flagbackup = F)
  
  flagdata(vis = 'uid___A002_Xb56f17_X164.ms',
    mode = 'manual',
    intent = '*POINTING*,*ATMOSPHERE*',
    flagbackup = F)
  
  flagcmd(vis = 'uid___A002_Xb56f17_X164.ms',
    inpmode = 'table',
    useapplied = True,
    action = 'plot',
    plotfile = 'uid___A002_Xb56f17_X164.ms.flagcmd.png')
  
  flagcmd(vis = 'uid___A002_Xb56f17_X164.ms',
    inpmode = 'table',
    useapplied = True,
    action = 'apply')
  

# Generation and time averaging of the WVR cal table
mystep = 4
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  os.system('rm -rf uid___A002_Xb56f17_X164.ms.wvr') 
  
  os.system('rm -rf uid___A002_Xb56f17_X164.ms.wvrgcal') 
  
  mylogfile = casalog.logfile()
  casalog.setlogfile('uid___A002_Xb56f17_X164.ms.wvrgcal')
  
  wvrgcal(vis = 'uid___A002_Xb56f17_X164.ms',
    caltable = 'uid___A002_Xb56f17_X164.ms.wvr',
    spw = [25, 27, 29, 31],
    smooth = '6.048s',
    toffset = 0,
    tie = ['Arp_220,J1516+1932', 'J1532+2344,J1516+1932'],
    statsource = 'Arp_220')
  
  casalog.setlogfile(mylogfile)
  
  if applyonly != True: aU.plotWVRSolutions(caltable='uid___A002_Xb56f17_X164.ms.wvr', spw='25', antenna='DA61',
    yrange=[-199,199],subplot=22, interactive=False, unwrap=True,
    figfile='uid___A002_Xb56f17_X164.ms.wvr.plots/uid___A002_Xb56f17_X164.ms.wvr') 
  
  #Note: If you see wraps in these plots, try changing yrange or unwrap=True 
  #Note: If all plots look strange, it may be a bad WVR on the reference antenna.
  #      To check, you can set antenna='' to show all baselines.
  

# Generation of the Tsys cal table
mystep = 5
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  os.system('rm -rf uid___A002_Xb56f17_X164.ms.tsys') 
  gencal(vis = 'uid___A002_Xb56f17_X164.ms',
    caltable = 'uid___A002_Xb56f17_X164.ms.tsys',
    caltype = 'tsys')
  
  # Flagging edge channels
  
  flagdata(vis = 'uid___A002_Xb56f17_X164.ms.tsys',
    mode = 'manual',
    spw = '17:0~3;124~127,19:0~3;124~127,21:0~3;124~127,23:0~3;124~127',
    flagbackup = F)
  
  if applyonly != True: aU.plotbandpass(caltable='uid___A002_Xb56f17_X164.ms.tsys', overlay='time', 
    xaxis='freq', yaxis='amp', subplot=22, buildpdf=False, interactive=False,
    showatm=True,pwv='auto',chanrange='92.1875%',showfdm=True, showBasebandNumber=True, showimage=False, 
    field='', figfile='uid___A002_Xb56f17_X164.ms.tsys.plots.overlayTime/uid___A002_Xb56f17_X164.ms.tsys') 
  
  
  if applyonly != True: es.checkCalTable('uid___A002_Xb56f17_X164.ms.tsys', msName='uid___A002_Xb56f17_X164.ms', interactive=False) 
  

# Generation of the antenna position cal table
mystep = 6
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  # Position for antenna PM03 is derived from baseline run made on 2016-08-13 05:48:23.
  
  # Position for antenna DA48 is derived from baseline run made on 2016-08-13 05:48:23.
  
  # Position for antenna DA61 is derived from baseline run made on 2016-07-18 10:20:58.
  
  # Position for antenna DA45 is derived from baseline run made on 2016-07-18 10:20:58.
  
  # Position for antenna DA58 is derived from baseline run made on 2016-07-18 10:20:58.
  
  # Position for antenna DV12 is derived from baseline run made on 2016-07-18 10:20:58.
  
  # Position for antenna DV17 is derived from baseline run made on 2016-08-13 05:48:23.
  
  # Note: no baseline run found for antenna DA42.
  
  # Position for antenna DV01 is derived from baseline run made on 2016-07-18 10:20:58.
  
  # Position for antenna DV25 is derived from baseline run made on 2016-07-18 10:20:58.
  
  os.system('rm -rf uid___A002_Xb56f17_X164.ms.antpos') 
  gencal(vis = 'uid___A002_Xb56f17_X164.ms',
    caltable = 'uid___A002_Xb56f17_X164.ms.antpos',
    caltype = 'antpos',
    antenna = 'DA45,DA48,DA58,DA61,DV01,DV12,DV17,DV25,PM03',
  #  parameter = [0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0])
    parameter = [-3.43537e-05,-2.88472e-04,-1.79971e-04,-5.98000e-05,3.07317e-04,6.45374e-05,-1.55534e-04,-2.43115e-04,-2.49249e-04,-5.79858e-05,4.40323e-05,-8.42426e-05,-2.85645e-04,-1.04644e-04,-1.93872e-04,-2.77429e-05,-4.10315e-04,-2.02591e-04,-1.70558e-05,-5.49953e-04,-2.54563e-04,6.99428e-06,-2.07287e-04,-2.51460e-04,-5.90390e-05,-1.51507e-04,-9.10173e-05])
  
  
  # antenna x_offset y_offset z_offset total_offset baseline_date
  # DV17    -1.70558e-05   -5.49953e-04   -2.54563e-04    6.06252e-04      2016-08-13 05:48:23
  # DV12    -2.77429e-05   -4.10315e-04   -2.02591e-04    4.58444e-04      2016-07-18 10:20:58
  # DA58    -1.55534e-04   -2.43115e-04   -2.49249e-04    3.81341e-04      2016-07-18 10:20:58
  # DV01    -2.85645e-04   -1.04644e-04   -1.93872e-04    3.60735e-04      2016-07-18 10:20:58
  # DA45    -3.43537e-05   -2.88472e-04   -1.79971e-04    3.41740e-04      2016-07-18 10:20:58
  # DV25     6.99428e-06   -2.07287e-04   -2.51460e-04    3.25959e-04      2016-07-18 10:20:58
  # DA48    -5.98000e-05    3.07317e-04    6.45374e-05    3.19664e-04      2016-08-13 05:48:23
  # PM03    -5.90390e-05   -1.51507e-04   -9.10173e-05    1.86344e-04      2016-08-13 05:48:23
  # DA61    -5.79858e-05    4.40323e-05   -8.42426e-05    1.11346e-04      2016-07-18 10:20:58
  

# Application of the WVR, Tsys and antpos cal tables
mystep = 7
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  
  
  from recipes.almahelpers import tsysspwmap
  tsysmap = tsysspwmap(vis = 'uid___A002_Xb56f17_X164.ms', tsystable = 'uid___A002_Xb56f17_X164.ms.tsys', tsysChanTol = 1)
  
  
  
  applycal(vis = 'uid___A002_Xb56f17_X164.ms',
    field = '0',
    spw = '25,27,29,31',
    gaintable = ['uid___A002_Xb56f17_X164.ms.tsys', 'uid___A002_Xb56f17_X164.ms.wvr', 'uid___A002_Xb56f17_X164.ms.antpos'],
    gainfield = ['0', '', ''],
    interp = 'linear,linear',
    spwmap = [tsysmap,[],[]],
    calwt = T,
    flagbackup = F)
  
  
  
  applycal(vis = 'uid___A002_Xb56f17_X164.ms',
    field = '1',
    spw = '25,27,29,31',
    gaintable = ['uid___A002_Xb56f17_X164.ms.tsys', 'uid___A002_Xb56f17_X164.ms.wvr', 'uid___A002_Xb56f17_X164.ms.antpos'],
    gainfield = ['1', '', ''],
    interp = 'linear,linear',
    spwmap = [tsysmap,[],[]],
    calwt = T,
    flagbackup = F)
  
  
  
  applycal(vis = 'uid___A002_Xb56f17_X164.ms',
    field = '2',
    spw = '25,27,29,31',
    gaintable = ['uid___A002_Xb56f17_X164.ms.tsys', 'uid___A002_Xb56f17_X164.ms.wvr', 'uid___A002_Xb56f17_X164.ms.antpos'],
    gainfield = ['2', '', ''],
    interp = 'linear,linear',
    spwmap = [tsysmap,[],[]],
    calwt = T,
    flagbackup = F)
  
  
  
  # Note: J1532+2344 didn't have any Tsys measurement, so I used the one made on Arp_220. This is probably Ok.
  
  applycal(vis = 'uid___A002_Xb56f17_X164.ms',
    field = '3',
    spw = '25,27,29,31',
    gaintable = ['uid___A002_Xb56f17_X164.ms.tsys', 'uid___A002_Xb56f17_X164.ms.wvr', 'uid___A002_Xb56f17_X164.ms.antpos'],
    gainfield = ['4', '', ''],
    interp = 'linear,linear',
    spwmap = [tsysmap,[],[]],
    calwt = T,
    flagbackup = F)
  
  
  
  applycal(vis = 'uid___A002_Xb56f17_X164.ms',
    field = '4',
    spw = '25,27,29,31',
    gaintable = ['uid___A002_Xb56f17_X164.ms.tsys', 'uid___A002_Xb56f17_X164.ms.wvr', 'uid___A002_Xb56f17_X164.ms.antpos'],
    gainfield = ['4', '', ''],
    interp = 'linear,linear',
    spwmap = [tsysmap,[],[]],
    calwt = T,
    flagbackup = F)
  
  
  
  if applyonly != True: es.getCalWeightStats('uid___A002_Xb56f17_X164.ms') 
  

# Split out science SPWs and time average
mystep = 8
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  os.system('rm -rf uid___A002_Xb56f17_X164.ms.split') 
  os.system('rm -rf uid___A002_Xb56f17_X164.ms.split.flagversions') 
  
  split(vis = 'uid___A002_Xb56f17_X164.ms',
    outputvis = 'uid___A002_Xb56f17_X164.ms.split',
    datacolumn = 'corrected',
    spw = '25,27,29,31',
    keepflags = T)
  
  

print "# Calibration"

# Listobs, and save original flags
mystep = 9
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  os.system('rm -rf uid___A002_Xb56f17_X164.ms.split.listobs')
  listobs(vis = 'uid___A002_Xb56f17_X164.ms.split',
    listfile = 'uid___A002_Xb56f17_X164.ms.split.listobs')
  
  
  if not os.path.exists('uid___A002_Xb56f17_X164.ms.split.flagversions/Original.flags'):
    flagmanager(vis = 'uid___A002_Xb56f17_X164.ms.split',
      mode = 'save',
      versionname = 'Original')
  
  

# Initial flagging
mystep = 10
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  # Flagging shadowed data
  
  flagdata(vis = 'uid___A002_Xb56f17_X164.ms.split',
    mode = 'shadow',
    flagbackup = F)
  
  # Antenna with bad Tsys

  flagdata(vis = 'uid___A002_Xb56f17_X164.ms.split',
    mode = 'manual',
    antenna= 'DA58',
    flagbackup = F)
  


# Putting a model for the flux calibrator(s)
mystep = 11
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  setjy(vis = 'uid___A002_Xb56f17_X164.ms.split',
    standard = 'manual',
    field = 'J1550+0527',
    fluxdensity = [0.746566497614, 0, 0, 0],
    spix = -0.697256164751,
    reffreq = '185.153031135GHz')
  
  

# Save flags before bandpass cal
mystep = 12
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  
  flagmanager(vis = 'uid___A002_Xb56f17_X164.ms.split',
    mode = 'save',
    versionname = 'BeforeBandpassCalibration')
  
  

# Bandpass calibration
mystep = 13
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  os.system('rm -rf uid___A002_Xb56f17_X164.ms.split.ap_pre_bandpass') 
  
  gaincal(vis = 'uid___A002_Xb56f17_X164.ms.split',
    caltable = 'uid___A002_Xb56f17_X164.ms.split.ap_pre_bandpass',
    field = '0', # J1924-2914
    spw = '0:0~1919,1:0~1919,2:0~1919,3:0~1919',
    scan = '1,3',
    solint = 'int',
    refant = 'DA61',
    calmode = 'p')
  
  if applyonly != True: es.checkCalTable('uid___A002_Xb56f17_X164.ms.split.ap_pre_bandpass', msName='uid___A002_Xb56f17_X164.ms.split', interactive=False) 
  
  os.system('rm -rf uid___A002_Xb56f17_X164.ms.split.bandpass') 
  bandpass(vis = 'uid___A002_Xb56f17_X164.ms.split',
    caltable = 'uid___A002_Xb56f17_X164.ms.split.bandpass',
    field = '0', # J1924-2914
    scan = '1,3',
    solint = 'inf',
    combine = 'scan',
    refant = 'DA61',
    solnorm = True,
    bandtype = 'B',
    gaintable = 'uid___A002_Xb56f17_X164.ms.split.ap_pre_bandpass')
  
  os.system('rm -rf uid___A002_Xb56f17_X164.ms.split.bandpass_smooth20ch') 
  
  bandpass(vis = 'uid___A002_Xb56f17_X164.ms.split',
    caltable = 'uid___A002_Xb56f17_X164.ms.split.bandpass_smooth20ch',
    field = '0', # J1924-2914
    scan = '1,3',
    solint = 'inf,8MHz',
    combine = 'scan',
    refant = 'DA61',
    solnorm = True,
    bandtype = 'B',
    gaintable = 'uid___A002_Xb56f17_X164.ms.split.ap_pre_bandpass')
  
  if applyonly != True: es.checkCalTable('uid___A002_Xb56f17_X164.ms.split.bandpass_smooth20ch', msName='uid___A002_Xb56f17_X164.ms.split', interactive=False) 
  
  if applyonly != True: es.checkCalTable('uid___A002_Xb56f17_X164.ms.split.bandpass', msName='uid___A002_Xb56f17_X164.ms.split', interactive=False) 
  

# Save flags before gain cal
mystep = 14
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  
  flagmanager(vis = 'uid___A002_Xb56f17_X164.ms.split',
    mode = 'save',
    versionname = 'BeforeGainCalibration')
  
  

# Gain calibration
mystep = 15
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  os.system('rm -rf uid___A002_Xb56f17_X164.ms.split.phase_int') 
  gaincal(vis = 'uid___A002_Xb56f17_X164.ms.split',
    caltable = 'uid___A002_Xb56f17_X164.ms.split.phase_int',
    field = '0~3', # J1924-2914,J1550+0527,J1516+1932,J1532+2344
    solint = 'int',
    refant = 'DA61',
    gaintype = 'G',
    calmode = 'p',
    gaintable = 'uid___A002_Xb56f17_X164.ms.split.bandpass_smooth20ch')
  
  if applyonly != True: es.checkCalTable('uid___A002_Xb56f17_X164.ms.split.phase_int', msName='uid___A002_Xb56f17_X164.ms.split', interactive=False) 
  
  os.system('rm -rf uid___A002_Xb56f17_X164.ms.split.ampli_inf') 
  gaincal(vis = 'uid___A002_Xb56f17_X164.ms.split',
    caltable = 'uid___A002_Xb56f17_X164.ms.split.ampli_inf',
    field = '0~3', # J1924-2914,J1550+0527,J1516+1932,J1532+2344
    solint = 'inf',
    refant = 'DA61',
    gaintype = 'T',
    calmode = 'a',
    gaintable = ['uid___A002_Xb56f17_X164.ms.split.bandpass_smooth20ch', 'uid___A002_Xb56f17_X164.ms.split.phase_int'])
  
  if applyonly != True: es.checkCalTable('uid___A002_Xb56f17_X164.ms.split.ampli_inf', msName='uid___A002_Xb56f17_X164.ms.split', interactive=False) 
  
  os.system('rm -rf uid___A002_Xb56f17_X164.ms.split.flux_inf') 
  os.system('rm -rf uid___A002_Xb56f17_X164.ms.split.fluxscale') 
  mylogfile = casalog.logfile()
  casalog.setlogfile('uid___A002_Xb56f17_X164.ms.split.fluxscale')
  
  fluxscaleDict = fluxscale(vis = 'uid___A002_Xb56f17_X164.ms.split',
    caltable = 'uid___A002_Xb56f17_X164.ms.split.ampli_inf',
    fluxtable = 'uid___A002_Xb56f17_X164.ms.split.flux_inf',
    reference = '1') # J1550+0527
  
  casalog.setlogfile(mylogfile)
  
  if applyonly != True: es.fluxscale2(caltable = 'uid___A002_Xb56f17_X164.ms.split.ampli_inf', removeOutliers=True, msName='uid___A002_Xb56f17_X164.ms', writeToFile=True, preavg=10000)
  
  os.system('rm -rf uid___A002_Xb56f17_X164.ms.split.phase_inf') 
  gaincal(vis = 'uid___A002_Xb56f17_X164.ms.split',
    caltable = 'uid___A002_Xb56f17_X164.ms.split.phase_inf',
    field = '0~3', # J1924-2914,J1550+0527,J1516+1932,J1532+2344
    solint = 'inf',
    refant = 'DA61',
    gaintype = 'G',
    calmode = 'p',
    gaintable = 'uid___A002_Xb56f17_X164.ms.split.bandpass_smooth20ch')
  
  if applyonly != True: es.checkCalTable('uid___A002_Xb56f17_X164.ms.split.phase_inf', msName='uid___A002_Xb56f17_X164.ms.split', interactive=False) 
  

# Save flags before applycal
mystep = 16
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  
  flagmanager(vis = 'uid___A002_Xb56f17_X164.ms.split',
    mode = 'save',
    versionname = 'BeforeApplycal')
  
  

# Application of the bandpass and gain cal tables
mystep = 17
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  for i in ['0', '1']: # J1924-2914,J1550+0527
    applycal(vis = 'uid___A002_Xb56f17_X164.ms.split',
      field = str(i),
      gaintable = ['uid___A002_Xb56f17_X164.ms.split.bandpass_smooth20ch', 'uid___A002_Xb56f17_X164.ms.split.phase_int', 'uid___A002_Xb56f17_X164.ms.split.flux_inf'],
      gainfield = ['', i, i],
      interp = 'linear,linear',
      calwt = T,
      flagbackup = F)
  
  applycal(vis = 'uid___A002_Xb56f17_X164.ms.split',
    field = '2,3~4', # Arp_220,J1532+2344
    gaintable = ['uid___A002_Xb56f17_X164.ms.split.bandpass_smooth20ch', 'uid___A002_Xb56f17_X164.ms.split.phase_inf', 'uid___A002_Xb56f17_X164.ms.split.flux_inf'],
    gainfield = ['', '2', '2'], # J1516+1932
    interp = 'linear,linear',
    calwt = T,
    flagbackup = F)
  

# Split out corrected column
mystep = 18
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  os.system('rm -rf uid___A002_Xb56f17_X164.ms.split.cal') 
  os.system('rm -rf uid___A002_Xb56f17_X164.ms.split.cal.flagversions') 
  
  listOfIntents = ['CALIBRATE_BANDPASS#ON_SOURCE',
   'CALIBRATE_FLUX#ON_SOURCE',
   'CALIBRATE_PHASE#ON_SOURCE',
   'CALIBRATE_WVR#ON_SOURCE',
   'OBSERVE_CHECK_SOURCE#ON_SOURCE',
   'OBSERVE_TARGET#ON_SOURCE']
  
  split(vis = 'uid___A002_Xb56f17_X164.ms.split',
    outputvis = 'uid___A002_Xb56f17_X164.ms.split.cal',
    datacolumn = 'corrected',
    intent = ','.join(listOfIntents),
    keepflags = T)
  
  

# Save flags after applycal
mystep = 19
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  
  flagmanager(vis = 'uid___A002_Xb56f17_X164.ms.split.cal',
    mode = 'save',
    versionname = 'AfterApplycal')
  
  

